@@ -3,6 +3,7 @@
 #include <errno.h>
 #include <netdb.h>
 #include <stdio.h>
+#include <string.h>
 #include <stdlib.h>
 #include <sys/socket.h>
 #include <sys/types.h>
@@ -105,8 +106,13 @@ char * get_data(char *string)
 	msg->msg_iov = &iov;
 	sendmsg(socket_fd, msg, 0);
 
-	char *data = malloc(2048 * sizeof(char));
-	int len = recv(socket_fd, data, 2048, 0);
+	/*
+	 * This is just...horrible.  Mental note to replace this
+	 * With a select, ioctl to determine size, and malloc based
+	 * on that
+	 */
+	char *data = malloc(8192);
+	int len = recv(socket_fd, data, 8192, 0);
 	close(socket_fd);
 	data[len] = '\0';
 	return data;
@@ -115,39 +121,43 @@ char * get_data(char *string)
 void parse_setup(char *setup_data)
 {
 	char *token, *ptr;
+	int i,j;
+	char *copy;
 	if((setup_data == NULL) || (strlen(setup_data) == 0)) return;
-	char copy[strlen(setup_data) + 1];
-	strncpy(copy, setup_data, strlen(setup_data) + 1);
+	copy = strdup(setup_data);
+	if (!copy)
+		return;
+
 	setup.banned_irqs = NULL;
 	setup.banned_cpus = NULL;
 	token = strtok_r(copy, " ", &ptr);
 	if(strncmp(token, "SLEEP", strlen("SLEEP"))) goto out;
-	setup.sleep = strtol(strtok_r(ptr, " ", &ptr), NULL, 10);
-	token = strtok_r(ptr, " ", &ptr);
+	setup.sleep = strtol(strtok_r(NULL, " ", &ptr), NULL, 10);
+	token = strtok_r(NULL, " ", &ptr);
 	/* Parse banned IRQ data */
 	while(!strncmp(token, "IRQ", strlen("IRQ"))) {
 		irq_t *new_irq = malloc(sizeof(irq_t));
-		new_irq->vector = strtol(strtok_r(ptr, " ", &ptr), NULL, 10);
-		token = strtok_r(ptr, " ", &ptr);
+		new_irq->vector = strtol(strtok_r(NULL, " ", &ptr), NULL, 10);
+		token = strtok_r(NULL, " ", &ptr);
 		if(strncmp(token, "LOAD", strlen("LOAD"))) goto out;
-		new_irq->load = strtol(strtok_r(ptr, " ", &ptr), NULL, 10);
-		token = strtok_r(ptr, " ", &ptr);
+		new_irq->load = strtol(strtok_r(NULL, " ", &ptr), NULL, 10);
+		token = strtok_r(NULL, " ", &ptr);
 		if(strncmp(token, "DIFF", strlen("DIFF"))) goto out;
-		new_irq->diff = strtol(strtok_r(ptr, " ", &ptr), NULL, 10);
+		new_irq->diff = strtol(strtok_r(NULL, " ", &ptr), NULL, 10);
 		token = strtok_r(ptr, " ", &ptr);
 		if(strncmp(token, "CLASS", strlen("CLASS"))) goto out;
-		new_irq->class = strtol(strtok_r(ptr, " ", &ptr), NULL, 10);
+		new_irq->class = strtol(strtok_r(NULL, " ", &ptr), NULL, 10);
 		new_irq->is_banned = 1;
 		new_irq->assigned_to = NULL;
 		setup.banned_irqs = g_list_append(setup.banned_irqs, new_irq);
-		token = strtok_r(ptr, " ", &ptr);
+		token = strtok_r(NULL, " ", &ptr);
 	}
 
 	if(strncmp(token, "BANNED", strlen("BANNED"))) goto out;
-	token = strtok_r(ptr, " ", &ptr);
-	for(int i = strlen(token) - 1; i >= 0; i--) {
+	token = strtok_r(NULL, " ", &ptr);
+	for(i = strlen(token) - 1; i >= 0; i--) {
 		char *map = hex_to_bitmap(token[i]);
-		for(int j = 3; j >= 0; j--) {
+		for(j = 3; j >= 0; j--) {
 			if(map[j] == '1') {
 				uint64_t *banned_cpu = malloc(sizeof(uint64_t));
 				*banned_cpu = (4 * (strlen(token) - (i + 1)) + (4 - (j + 1)));
@@ -155,15 +165,15 @@ void parse_setup(char *setup_data)
 								banned_cpu);
 			}
 		}
+	
 	}
+	free(copy);
 	return;
 
 out: {
 	/* Invalid data presented */
-	char invalid_data[128];
-	snprintf(invalid_data, 128, "Invalid data sent. Unexpected token: %s\n",
-				token);
-	printf("%s\n", invalid_data);
+	printf("Invalid data sent.  Unexpected token: %s", token);
+	free(copy);
 	g_list_free(tree);
 	exit(1);
 }
@@ -226,55 +236,63 @@ void assign_cpu_mask(cpu_node_t *node, void *data __attribute__((unused)))
 
 void parse_into_tree(char *data)
 {
-	if((data == NULL) || (strlen(data) == 0)) return;
 	char *token, *ptr;
 	cpu_node_t *parent = NULL;
-	char copy[strlen(data) + 1];
-	strncpy(copy, data, strlen(data) + 1);
+	char *copy;
 	tree = NULL;
+	
+	if (!data || strlen(data) == 0)
+		return;
+
+	copy = strdup(data);
+	if (!copy)
+		return;
 
 	token = strtok_r(copy, " ", &ptr);
 	while(token != NULL) {
 		/* Parse node data */
-		if(strncmp(token, "TYPE", strlen("TYPE"))) goto out;
+		if(strncmp(token, "TYPE", strlen("TYPE"))) {
+			free(copy);
+			 goto out;
+		}
 		cpu_node_t *new = malloc(sizeof(cpu_node_t));
 		new->irqs = NULL;
 		new->children = NULL;
 		new->cpu_list = NULL;
 		new->cpu_mask = NULL;
-		new->type = strtol(strtok_r(ptr, " ", &ptr), NULL, 10);
+		new->type = strtol(strtok_r(NULL, " ", &ptr), NULL, 10);
 		if(new->type == OBJ_TYPE_NODE) {
 			parent = NULL;
 		} else if(new->type >= parent->type) {
 			parent = parent->parent;
 		}
-		token = strtok_r(ptr, " ", &ptr);
+		token = strtok_r(NULL, " ", &ptr);
 		if(strncmp(token, "NUMBER", strlen("NUMBER"))) goto out;
-		new->number = strtol(strtok_r(ptr, " ", &ptr), NULL, 10);
-		token = strtok_r(ptr, " ", &ptr);
+		new->number = strtol(strtok_r(NULL, " ", &ptr), NULL, 10);
+		token = strtok_r(NULL, " ", &ptr);
 		if(strncmp(token, "LOAD", strlen("LOAD"))) goto out;
-		new->load = strtol(strtok_r(ptr, " ", &ptr), NULL, 10);
-		token = strtok_r(ptr, " ", &ptr);
+		new->load = strtol(strtok_r(NULL, " ", &ptr), NULL, 10);
+		token = strtok_r(NULL, " ", &ptr);
 		if(strncmp(token, "SAVE_MODE", strlen("SAVE_MODE"))) goto out;
-		new->is_powersave = strtol(strtok_r(ptr, " ", &ptr), NULL, 10);
-		token = strtok_r(ptr, " ", &ptr);
+		new->is_powersave = strtol(strtok_r(NULL, " ", &ptr), NULL, 10);
+		token = strtok_r(NULL, " ", &ptr);
 
 		/* Parse assigned IRQ data */
 		while((token != NULL) && (!strncmp(token, "IRQ", strlen("IRQ")))) {
 			irq_t *new_irq = malloc(sizeof(irq_t));
-			new_irq->vector = strtol(strtok_r(ptr, " ", &ptr), NULL, 10);
-			token = strtok_r(ptr, " ", &ptr);
+			new_irq->vector = strtol(strtok_r(NULL, " ", &ptr), NULL, 10);
+			token = strtok_r(NULL, " ", &ptr);
 			if(strncmp(token, "LOAD", strlen("LOAD"))) goto out;
-			new_irq->load = strtol(strtok_r(ptr, " ", &ptr), NULL, 10);
-			token = strtok_r(ptr, " ", &ptr);
+			new_irq->load = strtol(strtok_r(NULL, " ", &ptr), NULL, 10);
+			token = strtok_r(NULL, " ", &ptr);
 			if(strncmp(token, "DIFF", strlen("DIFF"))) goto out;
-			new_irq->diff = strtol(strtok_r(ptr, " ", &ptr), NULL, 10);
-			token = strtok_r(ptr, " ", &ptr);
+			new_irq->diff = strtol(strtok_r(NULL, " ", &ptr), NULL, 10);
+			token = strtok_r(NULL, " ", &ptr);
 			if(strncmp(token, "CLASS", strlen("CLASS"))) goto out;
-			new_irq->class = strtol(strtok_r(ptr, " ", &ptr), NULL, 10);
+			new_irq->class = strtol(strtok_r(NULL, " ", &ptr), NULL, 10);
 			new_irq->is_banned = 0;
 			new->irqs = g_list_append(new->irqs, new_irq);
-			token = strtok_r(ptr, " ", &ptr);
+			token = strtok_r(NULL, " ", &ptr);
 		}
 
 		if((token == NULL) || (strncmp(token, "IRQ", strlen("IRQ")))) {
@@ -289,17 +307,14 @@ void parse_into_tree(char *data)
 			}
 		}
 	}
-
+	free(copy);
 	for_each_node(tree, assign_cpu_lists, NULL);
 	for_each_node(tree, assign_cpu_mask, NULL);
 	return;
 
 out: {
 	/* Invalid data presented */
-	char invalid_data[128];
-	snprintf(invalid_data, 128, "Invalid data sent. Unexpected token: %s\n",
-				token);
-	printf("%s\n", invalid_data);
+	printf("Invalid data sent.  Unexpected token: %s\n", token);
 	g_list_free(tree);
 	exit(1);
 }
@@ -314,7 +329,7 @@ gboolean rescan_tree(gpointer data __attribute__((unused)))
 	if(is_tree) {
 		display_tree();
 	}
-
+	free(setup_data);
 	return TRUE;
 }
 
@@ -361,13 +376,13 @@ int main(int argc, char **argv)
 		DIR *dir = opendir("/proc");
 		if(dir) {
 			struct dirent *entry;
-			char cmdfile[64];
+			char cmdfile[512];
 			char cmdstring[256];
 			cmdstring[255] = '\0';
 			do {
 				entry = readdir(dir);
 				if(entry) {
-					snprintf(cmdfile, 64, "/proc/%s/cmdline", entry->d_name);
+					snprintf(cmdfile, 512, "/proc/%s/cmdline", entry->d_name);
 					FILE *f = fopen(cmdfile, "r");
 					if(f == NULL) {
 						continue;
@@ -397,4 +412,5 @@ int main(int argc, char **argv)
 
 	g_main_loop_quit(main_loop);
 	close_window(0);
+	return 0;
 }
